/*
 * xreplot - Function plotting utility
 * Copyright(c) David Drysdale 1995-1996
 * Freely distributable provided this copyright notice remains intact
 *
 * Send bug reports to dmd@lurklurk.org
 */

#include <stdio.h>
#include <stdlib.h>

/*****************************************************************************/
/* Allocation and utility routines                                           */
/*****************************************************************************/
#define NR_END 1
#define FREE_ARG char*

#define assert(C) do_assert(C, __FILE__, __LINE__)
void do_assert(int c, char *file, int line)
{
  if (!c) {
    fprintf(stderr, "Hit assert at %s in %d\n",file,line);
    exit(1);
  }
}

void nrerror(char error_text[])
/* Numerical Recipes standard error handler */
{
  fprintf(stderr,"Numerical Recipes run-time error...\n");
  fprintf(stderr,"%s\n",error_text);
  fprintf(stderr,"...now exiting to system...\n");
  exit(1);
}

float *vector(long nl, long nh)
/* allocate a float vector with subscript range v[nl..nh] */
{
  float *v;

  v=(float *)malloc((size_t) ((nh-nl+1+NR_END)*sizeof(float)));
  if (!v) nrerror("allocation failure in vector()");
  return v-nl+NR_END;
}

void free_vector(float *v, long nl, long nh)
/* free a float vector allocated with vector() */
{
  free((FREE_ARG) (v+nl-NR_END));
}

float **matrix(long nrl, long nrh, long ncl, long nch)
/* allocate a float matrix with subscript range m[nrl..nrh][ncl..nch] */
{
  long i, nrow=nrh-nrl+1,ncol=nch-ncl+1;
  float **m;

  /* allocate pointers to rows */
  m=(float **) malloc((size_t)((nrow+NR_END)*sizeof(float*)));
  if (!m) nrerror("allocation failure 1 in matrix()");
  m += NR_END;
  m -= nrl;

  /* allocate rows and set pointers to them */
  m[nrl]=(float *) malloc((size_t)((nrow*ncol+NR_END)*sizeof(float)));
  if (!m[nrl]) nrerror("allocation failure 2 in matrix()");
  m[nrl] += NR_END;
  m[nrl] -= ncl;

  for(i=nrl+1;i<=nrh;i++) m[i]=m[i-1]+ncol;

  /* return pointer to array of pointers to rows */
  return m;
}
void free_matrix(float **m, long nrl, long nrh, long ncl, long nch)
/* free a float matrix allocated by matrix() */
{
  free((FREE_ARG) (m[nrl]+ncl-NR_END));
  free((FREE_ARG) (m+nrl-NR_END));
}

void HSBtoRGB(float H, float S, float V, float *pR, float *pG, float *pB)
{
  int i;
  float f,p,q,t;
  if (S==0.0) {
    *pR = V; *pG = V; *pB = V;
  } else {
    if (H>=1.0) H=0.0;
    i = (int)(6.0*H);
    f = (6.0*H) - (float)i;
    p = V*(1.0-S);
    q = V*(1.0-S*f);
    t = V*(1.0-S*(1.0-f));
    switch (i) {
      case 0: *pR = V; *pG = t; *pB = p; break;
      case 1: *pR = q; *pG = V; *pB = p; break;
      case 2: *pR = p; *pG = V; *pB = t; break;
      case 3: *pR = p; *pG = q; *pB = V; break;
      case 4: *pR = t; *pG = p; *pB = V; break;
      case 5: *pR = V; *pG = p; *pB = q; break;
    }
  }
}

